MY_ACCOUNT_LB_URL = "https://api.e2enetworks.com/myaccount/"
GPU_URL = "api/v1/gpu/"
BASE_GPU_URL = f"{MY_ACCOUNT_LB_URL}{GPU_URL}"
VALIDATED_SUCCESSFULLY = "Validated Successfully"
INVALID_CREDENTIALS = "Validated Failed Invalid APIkey of Toekn"
headers = {
            'Connection': 'keep-alive',
            'Content-Type': 'application/json',
            'Origin': 'https://thor-gpu.e2enetworks.net',
            'Referer': 'https://thor-gpu.e2enetworks.net/',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-site',
        }
MANAGED_STORAGE = "managed"
E2E_OBJECT_STORAGE = "e2e_s3"
BUCKET_TYPES = [MANAGED_STORAGE, E2E_OBJECT_STORAGE]
BUCKET_TYPES_HELP = {
    MANAGED_STORAGE: "To Create New Bucket",
    E2E_OBJECT_STORAGE: " To Use Existing Bucket"
}

FREE_USAGE = "free_usage"
PAID_USAGE = "paid_usage"
INSTANCE_TYPE = [FREE_USAGE, PAID_USAGE]
MODEL_TYPES = ['pytorch', 'triton']
S3_ENDPOINT = "objectstore.e2enetworks.net"
hash_code_to_image_id = {
    "QHeMSAyC": 1,
    "skqbtZpd": 2,
    "NynWPeEJ": 3,
    "XEFKbStA": 4,
    "QPSKEFKQ": 5,
    "siGeSJyO": 6,
    "yYkVtTsC": 7,
    "wVOqCnpB": 8,
    "SICGgpoZ": 9,
    "ddUybAyh": 10,
    "yWNKwMlg": 11,
    "WtZaGkNZ": 12,
}
